"use strict";
// Copyright (c) Mito
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
require("../../css/column-header.css");
const gridStyling_1 = require("../utils/gridStyling");
const FilterIcons_1 = require("./icons/FilterIcons");
const ControlPanelTaskpane_1 = require("./taskpanes/ControlPanel/ControlPanelTaskpane");
const taskpanes_1 = require("./taskpanes/taskpanes");
/*
  A custom component that AG-Grid displays for the column
  header, that we extend to open the column header popup when clicked.

  NOTE: When you add a prop to this component, you must also pass it as a header
  prop, wherever the column is constructed.
*/
const ColumnHeader = (props) => {
    /*
      we split each word of the header (separated by _) into its own span element, adding the _ to
      the latter half of the split. ie: first_name -> [first, _name]
      This lets us wrap the column headers without breaking words in half, which we would
      do if we just used a fix length cutoff for each row as the cutoff.

      Note: this behaves better when the header's words are <10 characters, but worse when
      the header's words are > 10 characters. The 10 number is decided by the default column width
    */
    const wordsSpans = gridStyling_1.getHeaderWords(props.displayName).map(word => {
        // give the span a random key. we don't use the word because there may be duplicates
        return (react_1.default.createElement("span", { key: Math.random() }, word));
    });
    return (react_1.default.createElement("div", { className: 'column-header-cell', onClick: () => {
            props.setSelectedColumn(props.displayName);
        } },
        react_1.default.createElement("div", { className: 'column-header-header-container' },
            react_1.default.createElement("div", { onClick: () => {
                    props.setCurrOpenTaskpane({
                        type: taskpanes_1.TaskpaneType.CONTROL_PANEL,
                        columnHeader: props.displayName,
                        openEditingColumnHeader: true
                    });
                }, className: "column-header-column-header mr-1" }, wordsSpans)),
        react_1.default.createElement("div", { className: 'column-header-filter-container' },
            react_1.default.createElement("div", { className: 'column-header-filter-button', onClick: () => {
                    props.setCurrOpenTaskpane({
                        type: taskpanes_1.TaskpaneType.CONTROL_PANEL,
                        columnHeader: props.displayName,
                        openEditingColumnHeader: false,
                    });
                    props.setSelectedColumnControlPanelTab(ControlPanelTaskpane_1.ControlPanelTab.FilterSort);
                } },
                props.filters.length === 0 &&
                    react_1.default.createElement(FilterIcons_1.FilterEmptyIcon, null),
                props.filters.length !== 0 &&
                    react_1.default.createElement(FilterIcons_1.FilterNonEmptyIcon, null)))));
};
exports.default = ColumnHeader;
//# sourceMappingURL=ColumnHeader.js.map