"use strict";
// Copyright (c) Mito
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const Mito_1 = require("../Mito");
const BlueMitoFolk_1 = __importDefault(require("../icons/mitofolks/BlueMitoFolk"));
const PinkMitoFolk_1 = __importDefault(require("../icons/mitofolks/PinkMitoFolk"));
const YellowMitoFolk_1 = __importDefault(require("../icons/mitofolks/YellowMitoFolk"));
const FrustrationSelection_1 = __importDefault(require("../taskpanes/Feedback/FrustrationSelection"));
require("../../../css/signup-modal.css");
/*
    This file contains all the screens used in the signup modal. As these
    are only used in this one file, we keep them together for cleanlyness.
*/
/* Step one requires an email input */
const StepOne = (props) => {
    // Make sure the input is focused on, as we want the experience of sign up
    // to be as fluid as possible, and the autofocus does not work for this 
    // input element, for some reason
    const emailInputRef = react_1.useRef(null);
    react_1.useEffect(() => {
        var _a;
        (_a = emailInputRef.current) === null || _a === void 0 ? void 0 : _a.focus();
    });
    const onSubmit = (e) => __awaiter(void 0, void 0, void 0, function* () {
        e.preventDefault();
        yield props.mitoAPI.sendSignUp(props.email);
        props.next();
    });
    return (react_1.default.createElement("div", { className: 'signup-modal-left-column' },
        react_1.default.createElement("div", null,
            react_1.default.createElement("h1", { className: 'mt-0' }, "Sign Up for Mito"),
            react_1.default.createElement("p", { className: 'signup-modal-text' }, "We\u2019ll send you periodic product updates and welcome any feedback. And no spam. Duh.")),
        react_1.default.createElement("form", { className: 'signup-modal-email-form', onSubmit: onSubmit },
            react_1.default.createElement("h3", { className: 'mb-20px' }, "Your Email"),
            react_1.default.createElement("input", { ref: emailInputRef, className: 'input', type: 'email', value: props.email, onChange: (event) => { props.setEmail(event.target.value); }, placeholder: 'example@gmail.com', required: true, autoFocus: true }),
            react_1.default.createElement("button", { className: 'signup-modal-button-signup', type: 'submit' }, " Sign Up"))));
};
/* Step one requires the user to read the privacy policy */
const StepTwo = (props) => {
    return (react_1.default.createElement("div", { className: 'signup-modal-left-column' },
        react_1.default.createElement("div", null,
            react_1.default.createElement("h1", { className: 'mt-0' }, "Mito is Built for Privacy"),
            react_1.default.createElement("p", { className: 'signup-modal-text' },
                "We make sure none of your private data leaves your computer. Read our CCPA compliant privacy policy ",
                react_1.default.createElement("a", { href: 'https://privacy.trymito.io/privacy-policy', target: '_blank', rel: "noreferrer" },
                    react_1.default.createElement("u", null, "here")),
                ".")),
        react_1.default.createElement("div", { className: 'signup-modal-buttons' },
            react_1.default.createElement("button", { className: 'signup-modal-button-back', onClick: props.back }, " Back"),
            react_1.default.createElement("button", { className: 'signup-modal-button-next', onClick: props.next, autoFocus: true }, " Accept"))));
};
var QuestionAnswers;
(function (QuestionAnswers) {
    QuestionAnswers["PIVOT"] = "Pivot";
    QuestionAnswers["MERGE"] = "Merge";
    QuestionAnswers["COLUMN_FORMULAS"] = "Column Formulas";
    QuestionAnswers["EXPLORE_DATA"] = "Explore Datasets";
    QuestionAnswers["FILTER_SORT"] = "Filter/Sort";
    QuestionAnswers["NOT_SURE"] = "I'm Not Sure";
})(QuestionAnswers = exports.QuestionAnswers || (exports.QuestionAnswers = {}));
/* Step three asks the user want they want to use Mito for */
const StepThree = (props) => {
    const [answers, setAnswers] = react_1.useState([]);
    const InstallationQuestion = 'How was installation?';
    const [selectedFrustrationRating, setSelectedFrustrationRating] = react_1.useState(undefined);
    // Takes an answer and either selects or unselects it
    const toggleAnswer = (answer) => {
        const newAnswers = [...answers];
        if (answers.includes(answer)) {
            newAnswers.splice(newAnswers.indexOf(answer), 1);
            setAnswers(newAnswers);
        }
        else {
            newAnswers.push(answer);
            setAnswers(newAnswers);
        }
    };
    const onSubmit = () => __awaiter(void 0, void 0, void 0, function* () {
        // Set the intended behavior
        yield props.mitoAPI.sendIntendedBehavior(answers);
        // Set the feedback for installation
        yield props.mitoAPI.sendFeedback({ [InstallationQuestion]: selectedFrustrationRating });
        // Advance to the next step
        props.next();
    });
    return (react_1.default.createElement("div", { className: 'signup-modal-left-column' },
        react_1.default.createElement("div", null,
            react_1.default.createElement("h3", { className: 'mt-0 mb-5px' }, "What are you looking to do with Mito?"),
            react_1.default.createElement("div", { className: 'signup-modal-usage-grid' }, Object.values(QuestionAnswers).map((answer) => {
                const isSelected = answers.includes(answer);
                return (react_1.default.createElement("div", { className: 'signup-modal-usage-answer', style: { backgroundColor: isSelected ? 'pink' : 'white', border: isSelected ? '2px solid black' : '2px solid grey' }, key: answer, onClick: () => { toggleAnswer(answer); } }, answer));
            })),
            react_1.default.createElement("h3", { className: 'mt-5px mb-5px' }, "How easy was installation?"),
            react_1.default.createElement(FrustrationSelection_1.default, { selectedFrustrationRating: selectedFrustrationRating, setSelectedFrustrationRating: setSelectedFrustrationRating, padding: '2px 5px' })),
        react_1.default.createElement("div", { className: 'signup-modal-buttons' },
            react_1.default.createElement("button", { className: 'signup-modal-button-back', onClick: props.back }, " Back"),
            react_1.default.createElement("button", { className: 'signup-modal-button-next', onClick: onSubmit, autoFocus: true }, " Next"))));
};
/*
    Main signup modal, which collects the users email, shows them the privacy policy,
    and then asks them what they want to do with the tool.
*/
const SignupModal = (props) => {
    const [step, setStep] = react_1.useState(1);
    const [email, setEmail] = react_1.useState('');
    const next = () => {
        if (step + 1 > 3) {
            props.setModal({ type: Mito_1.ModalEnum.None });
            void props.mitoAPI.sendLogMessage('finished_signup');
        }
        else {
            const newStep = Math.min(step + 1, 3);
            setStep(newStep);
            void props.mitoAPI.sendLogMessage('switched_signup_step', {
                'old_signup_step': step,
                'new_signup_step': newStep
            });
        }
    };
    const back = () => {
        const newStep = Math.max(step - 1, 0);
        setStep(newStep);
        void props.mitoAPI.sendLogMessage('switched_signup_step', {
            'old_signup_step': step,
            'new_signup_step': newStep
        });
    };
    // Background colors of the different steps right column
    const backgroundColors = {
        1: '#FFEBEB',
        2: '#FFDAAE',
        3: '#F0C5BB',
    };
    return (react_1.default.createElement("div", { className: 'overlay' },
        react_1.default.createElement("div", { className: 'signup-modal-container' },
            react_1.default.createElement("div", { className: 'signup-modal-left-column-container' },
                step === 1 &&
                    react_1.default.createElement(StepOne, { next: next, email: email, setEmail: setEmail, mitoAPI: props.mitoAPI }),
                step === 2 &&
                    react_1.default.createElement(StepTwo, { next: next, back: back }),
                step === 3 &&
                    react_1.default.createElement(StepThree, { next: next, back: back, mitoAPI: props.mitoAPI })),
            react_1.default.createElement("div", { className: 'signup-modal-right-column-container', style: { backgroundColor: backgroundColors[step] } },
                step === 1 &&
                    react_1.default.createElement(BlueMitoFolk_1.default, null),
                step === 2 &&
                    react_1.default.createElement(YellowMitoFolk_1.default, null),
                step === 3 &&
                    react_1.default.createElement(PinkMitoFolk_1.default, null)))));
};
exports.default = SignupModal;
//# sourceMappingURL=SignupModal.js.map