"use strict";
// Copyright (c) Mito
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
// import css
require("../../../../css/column-header-modal.css");
require("../../../../css/column-name-card.css");
const TaskpaneError_1 = __importDefault(require("../TaskpaneError"));
const taskpanes_1 = require("../taskpanes");
const renameUtils_1 = require("./renameUtils");
/*
    A modal that allows a user to sort a column
*/
class ColumnNameCard extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.updateUnsubmittedNewColumnHeader = (e) => {
            const newColumnHeader = e.target.value;
            // Update the saved unsubmittedColumnHeader
            this.setState({
                unsubmittedNewColumnHeader: newColumnHeader
            });
            // Display an error message, if it has one
            this.setInvalidInputError(renameUtils_1.getHeaderErrorMessage(newColumnHeader));
        };
        this.changeColumnHeader = () => __awaiter(this, void 0, void 0, function* () {
            if (!renameUtils_1.isValidHeader(this.state.unsubmittedNewColumnHeader)) {
                // We make sure the new column header is a valid mito column,
                // and we don't allow the user to submit if it's not! 
                // We also update the error to tell them to fix the issue.
                this.setInvalidInputError(`Please fix issues before submitting: ${renameUtils_1.getHeaderErrorMessage(this.state.unsubmittedNewColumnHeader)}`);
                return;
            }
            // If the column header did not change, just stop renaming without updating the name to avoid error
            if (this.state.columnHeader == this.state.unsubmittedNewColumnHeader) {
                this.setState({
                    isRename: false
                });
                return;
            }
            const stepID = yield this.props.mitoAPI.sendRenameColumn(this.props.selectedSheetIndex, this.state.columnHeader, this.state.unsubmittedNewColumnHeader, this.state.stepID);
            this.setState(prevState => {
                return {
                    stepID: stepID,
                    isRename: false,
                    columnHeader: prevState.unsubmittedNewColumnHeader,
                    unsubmittedNewColumnHeader: prevState.unsubmittedNewColumnHeader
                };
            });
            this.props.setCurrOpenTaskpane({ type: taskpanes_1.TaskpaneType.CONTROL_PANEL, columnHeader: this.state.columnHeader, openEditingColumnHeader: false });
        });
        this.state = {
            columnHeader: props.columnHeader,
            unsubmittedNewColumnHeader: props.columnHeader,
            invalidInputError: '',
            isRename: props.openEditingColumnHeader,
            stepID: ''
        };
        this.setColumnHeader = this.setColumnHeader.bind(this);
        this.updateUnsubmittedNewColumnHeader = this.updateUnsubmittedNewColumnHeader.bind(this);
        this.setInvalidInputError = this.setInvalidInputError.bind(this);
        this.setIsRename = this.setIsRename.bind(this);
    }
    setColumnHeader(columnHeader) {
        this.setState({
            columnHeader: columnHeader
        });
    }
    setInvalidInputError(errorMessage) {
        this.setState({
            invalidInputError: errorMessage
        });
    }
    setIsRename(isRename) {
        this.setState(prevState => {
            return {
                isRename: isRename,
                unsubmittedNewColumnHeader: prevState.columnHeader
            };
        });
    }
    render() {
        return (react_1.default.createElement("div", { className: 'column-name-card-container' },
            react_1.default.createElement("div", { className: 'default-taskpane-header-div' },
                react_1.default.createElement("div", { className: 'column-name-card-column-name-div', onClick: () => {
                        if (!this.state.isRename) {
                            this.setIsRename(!this.state.isRename);
                            this.setInvalidInputError('');
                        }
                    } },
                    this.state.isRename &&
                        react_1.default.createElement("form", { onSubmit: (e) => { e.preventDefault(); void this.changeColumnHeader(); }, onBlur: (e) => { e.preventDefault(); void this.changeColumnHeader(); } },
                            react_1.default.createElement("input", { type: 'text', className: 'column-name-card-input-text', value: this.state.unsubmittedNewColumnHeader, onChange: (e) => { this.updateUnsubmittedNewColumnHeader(e); }, autoFocus: true })),
                    !this.state.isRename &&
                        react_1.default.createElement("p", { className: 'column-name-card-column-header' }, this.state.columnHeader)),
                react_1.default.createElement("div", { className: 'default-taskpane-header-exit-button-div', onClick: () => this.props.setCurrOpenTaskpane({ type: taskpanes_1.TaskpaneType.NONE }) },
                    react_1.default.createElement("svg", { width: "18", height: "18", viewBox: "0 0 13 13", fill: "none", xmlns: "http://www.w3.org/2000/svg" },
                        react_1.default.createElement("line", { x1: "0.707107", y1: "1.29289", x2: "11.3136", y2: "11.8994", stroke: "#343434", strokeWidth: "2" }),
                        react_1.default.createElement("line", { x1: "0.7072", y1: "11.8995", x2: "11.3137", y2: "1.29297", stroke: "#343434", strokeWidth: "2" })))),
            react_1.default.createElement(TaskpaneError_1.default, { message: this.state.invalidInputError })));
    }
}
exports.default = ColumnNameCard;
//# sourceMappingURL=ColumnNameCard.js.map