"use strict";
// Copyright (c) Mito
// Distributed under the terms of the Modified BSD License.
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const DefaultTaskpane_1 = __importDefault(require("../DefaultTaskpane"));
require("../../../../css/feedback-taskpane.css");
// Import 
const taskpanes_1 = require("../taskpanes");
const NPS_1 = __importDefault(require("./NPS"));
const FrustrationSelection_1 = __importDefault(require("./FrustrationSelection"));
// Keeps track of where you access feedback from
// so we know what is effective for getting feedback
// from users
var FeedbackOpenLocation;
(function (FeedbackOpenLocation) {
    FeedbackOpenLocation["AUTO_POPUP"] = "Automatically Popped Up";
    FeedbackOpenLocation["FROM_DOCUMENTATION"] = "From Documentation";
})(FeedbackOpenLocation = exports.FeedbackOpenLocation || (exports.FeedbackOpenLocation = {}));
/*
    Taskpane that allows a user to submit feedback on their
    usage of Mito so far
*/
function FeedbackTaskpane(props) {
    const NPSQuestion = 'How likely are you to reccomend Mito to a colleague or friend?';
    const [selectedNPS, setSelectedNPS] = react_1.useState(undefined);
    const ImprovementsQuestion = 'What can we add or change to improve your experience with Mito?';
    const [improvementText, setImprovementText] = react_1.useState('');
    const FrustrationQuestion = 'It was frustrating to find the functionality I wanted to use.';
    const [selectedFrustrationRating, setSelectedFrustrationRating] = react_1.useState(undefined);
    // Wrap the setter for selectedNPS so that we can log changes easily
    const changeSelectedNPS = (newSelectedNPS) => {
        // We log that the user switched
        void props.mitoAPI.sendLogMessage('changed_nps_score', {
            'old_selected_nps': selectedNPS,
            'new_selected_nps': newSelectedNPS,
        });
        // Then we actually set the value
        setSelectedNPS(newSelectedNPS);
    };
    const submitFeedback = () => {
        // Send a message to the backend with this feedback
        void props.mitoAPI.sendFeedback({
            'opened_location': props.openLocation,
            [NPSQuestion]: selectedNPS,
            [ImprovementsQuestion]: improvementText,
            [FrustrationQuestion]: selectedFrustrationRating
        });
        // Then, advance to the feedback thanks taskpane, to thank them
        // because we really seriously do appreciate the feedback!
        props.setCurrOpenTaskpane({ type: taskpanes_1.TaskpaneType.FEEDBACK_THANKS });
    };
    react_1.useEffect(() => {
        // We log where we opened it from, making sure to only run this once
        void props.mitoAPI.sendLogMessage('opened_feedback', { 'open_location': props.openLocation });
    }, []);
    return (react_1.default.createElement(DefaultTaskpane_1.default, { header: 'How is Mito so far?', setCurrOpenTaskpane: props.setCurrOpenTaskpane, taskpaneBody: react_1.default.createElement(react_1.Fragment, null,
            react_1.default.createElement("p", { className: 'mb-5px txt-15' }, NPSQuestion),
            react_1.default.createElement(NPS_1.default, { selectedNPS: selectedNPS, setSelectedNPS: changeSelectedNPS }),
            react_1.default.createElement("p", { className: 'mb-5px mt-15px txt-15' }, ImprovementsQuestion),
            react_1.default.createElement("textarea", { className: 'feedback-taskpane-textarea', value: improvementText, onChange: (event) => { setImprovementText(event.target.value); } }),
            react_1.default.createElement("p", { className: 'mb-5px mt-15px txt-15' }, FrustrationQuestion),
            react_1.default.createElement(FrustrationSelection_1.default, { selectedFrustrationRating: selectedFrustrationRating, setSelectedFrustrationRating: setSelectedFrustrationRating, justifyContent: 'space-between' }),
            react_1.default.createElement("button", { className: 'blue-button txt-16', onClick: submitFeedback }, "Send Feedback")), callbackOnClose: () => {
            // If the feedback is closed, we set that the user closed the feedback,
            void props.mitoAPI.setClosedFeedback();
            // And we also log that they closed the feedback, including where they got it from
            void props.mitoAPI.sendLogMessage('closed_feedback', { 'open_location': props.openLocation });
        } }));
}
exports.default = FeedbackTaskpane;
//# sourceMappingURL=FeedbackTaskpane.js.map