"use strict";
// Copyright (c) Mito
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const DefaultTaskpane_1 = __importDefault(require("../DefaultTaskpane"));
// Import 
require("../../../../css/import-taskpane.css");
const LargeSelect_1 = __importDefault(require("../../elements/LargeSelect"));
const XIcon_1 = __importDefault(require("../../icons/XIcon"));
const ImportByPathCard_1 = __importDefault(require("./ImportByPathCard"));
/*
    The FileImportType is used to tell the UI which import type to display:
    - CURRENT_FOLDER => display select import
    - BY_PATH => input field for paths.
*/
var FileImportType;
(function (FileImportType) {
    FileImportType["CURRENT_FOLDER"] = "current_folder";
    FileImportType["BY_PATH"] = "by_path";
})(FileImportType = exports.FileImportType || (exports.FileImportType = {}));
/*
    Provides a live updating import modal
    that allows users to import data
*/
function ImportTaskpane(props) {
    // Save the step id for overwriting
    const [stepID, setStepID] = react_1.useState('');
    // The datafiles are the files loaded from the backend
    const [dataFiles, setDataFiles] = react_1.useState([]);
    const [hasLoadedDataFiles, setHasLoadedDataFiles] = react_1.useState(false);
    // Which files are actually selected by the user
    const [selectedFiles, setSelectedFiles] = react_1.useState([]);
    /*
        Sends a message to the backend, importing files (and overwriting
        the step if it already exists).
    */
    function sendImportMessage(files) {
        return __awaiter(this, void 0, void 0, function* () {
            // Collect all of the file paths to submit 
            let filePaths = files.map(file => file.path);
            // Remove any empty file paths
            filePaths = filePaths.filter(filePath => filePath !== '');
            // Send message to backend to process import
            const newStepID = yield props.mitoAPI.sendSimpleImportMessage(filePaths, stepID);
            setStepID(newStepID);
        });
    }
    /*
        Loads the data files from the backend.
    */
    function getDataFiles() {
        return __awaiter(this, void 0, void 0, function* () {
            // Loads the data files
            const receivedDataFiles = yield props.mitoAPI.getDataFiles();
            setDataFiles(receivedDataFiles);
            // Save that we have loaded the files
            setHasLoadedDataFiles(true);
        });
    }
    // Updates a specific imported file to a specific import
    const updateImportedFile = (index, newImportPath, importType) => __awaiter(this, void 0, void 0, function* () {
        const newImports = [...selectedFiles];
        newImports[index] = {
            path: newImportPath,
            importType: importType
        };
        setSelectedFiles(newImports);
        // Set a message to actually update this
        yield sendImportMessage(newImports);
    });
    // Adds a new file to import by current folder select
    const addNewCurrentFolderFileImport = () => __awaiter(this, void 0, void 0, function* () {
        const newImports = [...selectedFiles];
        // We take the first file that has not been imported, or if all files have been
        // imported, than just the first file
        let newImport = dataFiles[0];
        const selectedFilePaths = selectedFiles.map(fileObj => fileObj.path);
        const unimportedFiles = dataFiles.filter(fileName => !selectedFilePaths.includes(fileName));
        if (unimportedFiles.length > 0) {
            newImport = unimportedFiles[0];
        }
        newImports.push({
            path: newImport,
            importType: FileImportType.CURRENT_FOLDER
        });
        setSelectedFiles(newImports);
        // Send a message to actually update this
        yield sendImportMessage(newImports);
    });
    // Adds a new file to import by path.
    // We ue it to create the UI input field
    const addNewByPathFileImport = () => {
        const newImports = [...selectedFiles];
        newImports.push({
            path: '',
            importType: FileImportType.BY_PATH
        });
        setSelectedFiles(newImports);
    };
    const setImportedPath = (index, path) => __awaiter(this, void 0, void 0, function* () {
        // Update the path of the submitted input field
        const newImports = [...selectedFiles];
        newImports[index].path = path;
        // Set the state
        setSelectedFiles(newImports);
        // Send a message to actually update this
        yield sendImportMessage(newImports);
    });
    // Removes a file from being imported
    const removeFileImport = (index) => __awaiter(this, void 0, void 0, function* () {
        const newImports = [...selectedFiles];
        newImports.splice(index, 1);
        setSelectedFiles(newImports);
        // Send a message to actually update this
        yield sendImportMessage(newImports);
    });
    react_1.useEffect(() => {
        // We load the data after the component renders
        void getDataFiles();
    }, []); // empty array is necessary to have this run only once
    return (react_1.default.createElement(DefaultTaskpane_1.default, { header: 'Import CSV Files', setCurrOpenTaskpane: props.setCurrOpenTaskpane, taskpaneBody: react_1.default.createElement(react_1.default.Fragment, null,
            !hasLoadedDataFiles &&
                // We display a loading message while loading the files from the backend
                react_1.default.createElement("div", null, "Loading data files in the current folder..."),
            hasLoadedDataFiles &&
                // If we loaded some files, we let users import files
                react_1.default.createElement(react_1.default.Fragment, null,
                    selectedFiles.map((fileObj, i) => {
                        if (fileObj.importType === FileImportType.CURRENT_FOLDER) {
                            return (react_1.default.createElement("div", { key: i, className: 'import-file-select-container' },
                                react_1.default.createElement(LargeSelect_1.default, { key: i, startingValue: fileObj.path, optionsArray: dataFiles, setValue: (fileName) => { void updateImportedFile(i, fileName, FileImportType.CURRENT_FOLDER); }, extraLarge: true }),
                                react_1.default.createElement("div", { className: 'import-file-delete', onClick: () => { void removeFileImport(i); } },
                                    react_1.default.createElement(XIcon_1.default, null))));
                        }
                        else {
                            return (
                            // See note below about key. 
                            react_1.default.createElement("div", { key: i + fileObj.path, className: 'import-file-select-container' },
                                react_1.default.createElement(ImportByPathCard_1.default
                                /*
                                    Without setting the key like this, when the user deletes a sheet,
                                    it always deletes the last import by path. We're not sure why this
                                    occurs, but making the keys unique in this way fixes it.
                                */
                                , { 
                                    /*
                                        Without setting the key like this, when the user deletes a sheet,
                                        it always deletes the last import by path. We're not sure why this
                                        occurs, but making the keys unique in this way fixes it.
                                    */
                                    key: i + fileObj.path, path: fileObj.path, setImportedPath: (path) => { void setImportedPath(i, path); } }),
                                react_1.default.createElement("div", { className: 'import-file-delete', onClick: () => { void removeFileImport(i); } },
                                    react_1.default.createElement(XIcon_1.default, null))));
                        }
                    }),
                    hasLoadedDataFiles && dataFiles.length === 0 &&
                        /* If there are no files in the Current Folder don't let the use click the Add File from Current Folder Button */
                        react_1.default.createElement("div", null,
                            react_1.default.createElement("div", { className: 'mb-5' },
                                "There are no files in the current folder to import. ",
                                react_1.default.createElement("br", null),
                                " ",
                                react_1.default.createElement("br", null),
                                "Upload files to this folder, and then reopen the import taskpane. ",
                                react_1.default.createElement("br", null),
                                " ",
                                react_1.default.createElement("br", null),
                                "You can read more about importing ",
                                react_1.default.createElement("a", { href: 'https://docs.trymito.io/getting-started/importing-data-to-mito', target: "_blank", rel: "noreferrer" },
                                    react_1.default.createElement("u", null, "here.")))),
                    hasLoadedDataFiles && dataFiles.length !== 0 &&
                        react_1.default.createElement("div", { className: 'import-add-file-from-current-folder-button', onClick: addNewCurrentFolderFileImport }, "+ Add File from Current Folder"),
                    react_1.default.createElement("div", { className: 'import-add-file-by-path-button', onClick: addNewByPathFileImport },
                        "or ",
                        react_1.default.createElement("u", null, "Add File by Path")))) }));
}
exports.default = ImportTaskpane;
//# sourceMappingURL=ImportTaskpane.js.map