"use strict";
// Copyright (c) Mito
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const DefaultTaskpane_1 = __importDefault(require("../DefaultTaskpane"));
const MultiToggleBox_1 = __importDefault(require("../../elements/MultiToggleBox"));
const mergeUtils_1 = require("./mergeUtils");
// Import 
require("../../../../css/pivot-taskpane.css");
const MergeSheetAndKeySelection_1 = __importDefault(require("./MergeSheetAndKeySelection"));
// Enum to allow you to refer to the first or second sheet by name, for clarity
var MergeSheet;
(function (MergeSheet) {
    MergeSheet[MergeSheet["First"] = 0] = "First";
    MergeSheet[MergeSheet["Second"] = 1] = "Second";
})(MergeSheet = exports.MergeSheet || (exports.MergeSheet = {}));
class MergeTaskpane extends react_1.default.Component {
    constructor(props) {
        super(props);
        const sheetOneIndex = 0;
        const sheetTwoIndex = Math.min(1, props.sheetJSONArray.length - 1);
        if (props.sheetJSONArray.length === 0) {
            // If there is no data, we just set default values
            this.state = {
                sheetOneIndex: sheetOneIndex,
                sheetTwoIndex: sheetTwoIndex,
                sheetOneMergeKey: '',
                sheetTwoMergeKey: '',
                sheetOneSelectedColumns: [],
                sheetTwoSelectedColumns: [],
                sheetOneToggleAll: true,
                sheetTwoToggleAll: true,
                originalDfNames: props.dfNames,
                stepID: ''
            };
        }
        else {
            const suggestedKeys = mergeUtils_1.getSuggestedColumnHeaderKeys(props.sheetJSONArray, sheetOneIndex, sheetTwoIndex);
            // We default to selecting _all_ columns
            const sheetOneSelectedColumns = [...this.props.sheetJSONArray[sheetOneIndex].columns];
            const sheetTwoSelectedColumns = [...this.props.sheetJSONArray[sheetTwoIndex].columns];
            this.state = {
                sheetOneIndex: sheetOneIndex,
                sheetTwoIndex: sheetTwoIndex,
                sheetOneMergeKey: suggestedKeys.sheetOneMergeKey,
                sheetTwoMergeKey: suggestedKeys.sheetTwoMergeKey,
                sheetOneSelectedColumns: sheetOneSelectedColumns,
                sheetTwoSelectedColumns: sheetTwoSelectedColumns,
                sheetOneToggleAll: true,
                sheetTwoToggleAll: true,
                originalDfNames: props.dfNames,
                stepID: ''
            };
        }
        this.setNewSheetIndex = this.setNewSheetIndex.bind(this);
        this.setNewMergeKey = this.setNewMergeKey.bind(this);
        this.toggleKeepColumnHeader = this.toggleKeepColumnHeader.bind(this);
        this.toggleAll = this.toggleAll.bind(this);
        this.sendMergeMessage = this.sendMergeMessage.bind(this);
    }
    /*
        It is best practice to make async calls not in the constructor,
        but rather in the componentDidMount lifecycle method!
    */
    componentDidMount() {
        // Send the first merge message there are at least 2 sheets in Mito.
        if (this.props.sheetJSONArray.length >= 2) {
            void this.sendMergeMessage();
        }
    }
    /*
        When one of the two merge indexes is changed, we change state by:
        1. Updating the sheet index.
        2. Updating the columns that are selected (this defaults to all), as well as the toggle
           for this sheet.
        3. Trying to find a new merge key between the sheets
    */
    setNewSheetIndex(sheetNumber, newSheetIndex) {
        const indexName = sheetNumber == MergeSheet.First ? 'sheetOneIndex' : 'sheetTwoIndex';
        const selectedColumnsName = sheetNumber == MergeSheet.First ? 'sheetOneSelectedColumns' : 'sheetTwoSelectedColumns';
        const toggleAllName = sheetNumber == MergeSheet.First ? 'sheetOneToggleAll' : 'sheetTwoToggleAll';
        const newSelectedColumns = [...this.props.sheetJSONArray[newSheetIndex].columns];
        this.setState(prevState => {
            // Return if we're not changing anything!
            if (prevState[indexName] == newSheetIndex) {
                return;
            }
            const newSuggestedKeys = mergeUtils_1.getSuggestedColumnHeaderKeys(this.props.sheetJSONArray, sheetNumber === MergeSheet.First ? newSheetIndex : prevState.sheetOneIndex, sheetNumber === MergeSheet.Second ? newSheetIndex : prevState.sheetTwoIndex);
            return Object.assign(Object.assign({}, prevState), { [indexName]: newSheetIndex, [selectedColumnsName]: newSelectedColumns, [toggleAllName]: true, sheetOneMergeKey: newSuggestedKeys.sheetOneMergeKey, sheetTwoMergeKey: newSuggestedKeys.sheetTwoMergeKey });
        }, () => {
            void this.sendMergeMessage();
        });
    }
    /*
        Sets a new merge key for one of the merge sheets
    */
    setNewMergeKey(sheetNumber, newMergeKey) {
        const mergeKeyName = sheetNumber === MergeSheet.First ? 'sheetOneMergeKey' : 'sheetTwoMergeKey';
        this.setState(prevState => {
            return Object.assign(Object.assign({}, prevState), { [mergeKeyName]: newMergeKey });
        }, () => {
            void this.sendMergeMessage();
        });
    }
    /*
        Toggles if we should keep a specific column at from one of the sheets in the new
        merged sheet.
    */
    toggleKeepColumnHeader(sheetNumber, columnHeader) {
        const selectedColumnsName = sheetNumber == MergeSheet.First ? 'sheetOneSelectedColumns' : 'sheetTwoSelectedColumns';
        const mergeKeyName = sheetNumber == MergeSheet.First ? 'sheetOneMergeKey' : 'sheetTwoMergeKey';
        this.setState(prevState => {
            // We the don't let you toggle the merge key!
            if (prevState[mergeKeyName] === columnHeader) {
                return;
            }
            const newSelectedColumns = [...prevState[selectedColumnsName]];
            if (newSelectedColumns.includes(columnHeader)) {
                newSelectedColumns.splice(newSelectedColumns.indexOf(columnHeader), 1);
            }
            else {
                newSelectedColumns.push(columnHeader);
            }
            return Object.assign(Object.assign({}, prevState), { [selectedColumnsName]: newSelectedColumns });
        }, () => {
            void this.sendMergeMessage();
        });
    }
    /*
        Toggles all the check boxes selected for a given sheet, based on the current
        toggleAll state for that sheet.
    */
    toggleAll(sheetNumber) {
        const sheetIndexName = sheetNumber === MergeSheet.First ? 'sheetOneIndex' : 'sheetTwoIndex';
        const toggleAllName = sheetNumber === MergeSheet.First ? 'sheetOneToggleAll' : 'sheetTwoToggleAll';
        const selectedColumnsName = sheetNumber === MergeSheet.First ? 'sheetOneSelectedColumns' : 'sheetTwoSelectedColumns';
        this.setState(prevState => {
            let newSelectedColumns = [];
            if (!prevState[toggleAllName]) {
                newSelectedColumns = [...this.props.sheetJSONArray[this.state[sheetIndexName]].columns];
            }
            return Object.assign(Object.assign({}, prevState), { [selectedColumnsName]: newSelectedColumns, [toggleAllName]: !prevState[toggleAllName] });
        }, () => {
            void this.sendMergeMessage();
        });
    }
    /*
        Completes the merge operation by sending information for the merge
        to the backend, potentially overwriting what's already there!
    */
    sendMergeMessage() {
        return __awaiter(this, void 0, void 0, function* () {
            // NOTE: We make sure to send the merge keys in the selected columns, no matter what
            const sheetOneSelectedColumns = [...this.state.sheetOneSelectedColumns];
            if (!sheetOneSelectedColumns.includes(this.state.sheetOneMergeKey)) {
                sheetOneSelectedColumns.push(this.state.sheetOneMergeKey);
            }
            const sheetTwoSelectedColumns = [...this.state.sheetTwoSelectedColumns];
            if (!sheetTwoSelectedColumns.includes(this.state.sheetTwoMergeKey)) {
                sheetTwoSelectedColumns.push(this.state.sheetTwoMergeKey);
            }
            const stepID = yield this.props.mitoAPI.sendMergeMessage(this.state.sheetOneIndex, this.state.sheetOneMergeKey, sheetOneSelectedColumns, this.state.sheetTwoIndex, this.state.sheetTwoMergeKey, sheetTwoSelectedColumns, this.state.stepID);
            // Save the merge ID
            this.setState({ stepID: stepID });
        });
    }
    render() {
        /*
            If there are less than 2 sheets in Mito to merge together,
            then display this error message.
        */
        if (this.props.sheetJSONArray.length < 2) {
            return (react_1.default.createElement(DefaultTaskpane_1.default, { header: 'Merge Sheets Together', setCurrOpenTaskpane: this.props.setCurrOpenTaskpane, taskpaneBody: react_1.default.createElement(react_1.Fragment, null, "Make sure you have at least two dataframes in Mito before merging.") }));
        }
        /*
            We don't let you select or unselect the sheet merge key, and note that we must account
            for the shift in the indexes that this causes when updating if the state of an item is
            toggled.

            Thus, we filter out the merge keys from both the list of columns, as well as the
            toggles for these columns.
        */
        const sheetOneOriginalColumns = this.props.sheetJSONArray[this.state.sheetOneIndex].columns;
        const sheetTwoOriginalColumns = this.props.sheetJSONArray[this.state.sheetTwoIndex].columns;
        const sheetOneColumns = sheetOneOriginalColumns.filter(columnHeader => columnHeader !== this.state.sheetOneMergeKey);
        const sheetTwoColumns = sheetTwoOriginalColumns.filter(columnHeader => columnHeader !== this.state.sheetTwoMergeKey);
        const sheetOneToggles = sheetOneColumns.map(columnHeader => this.state.sheetOneSelectedColumns.includes(columnHeader));
        const sheetTwoToggles = sheetTwoColumns.map(columnHeader => this.state.sheetTwoSelectedColumns.includes(columnHeader));
        return (react_1.default.createElement(DefaultTaskpane_1.default, { header: 'Merge Sheets Together', setCurrOpenTaskpane: this.props.setCurrOpenTaskpane, taskpaneBody: react_1.default.createElement(react_1.Fragment, null,
                react_1.default.createElement(MergeSheetAndKeySelection_1.default, { sheetNum: MergeSheet.First, dfNames: this.state.originalDfNames, sheetIndex: this.state.sheetOneIndex, otherSheetIndex: this.state.sheetTwoIndex, setNewSheetIndex: (newSheetIndex) => { this.setNewSheetIndex(MergeSheet.First, newSheetIndex); }, columnHeaders: this.props.sheetJSONArray[this.state.sheetOneIndex].columns, mergeKey: this.state.sheetOneMergeKey, setNewMergeKey: (newMergeKey) => this.setNewMergeKey(MergeSheet.First, newMergeKey) }),
                react_1.default.createElement("div", { className: 'mb-25px' },
                    react_1.default.createElement("p", { className: 'default-taskpane-body-section-title-text' }, "Columns to Keep"),
                    react_1.default.createElement(MultiToggleBox_1.default, { itemsArray: sheetOneColumns, itemToggleState: sheetOneToggles, toggleItemAtIndex: (index) => this.toggleKeepColumnHeader(MergeSheet.First, sheetOneColumns[index]), toggleAllOptions: {
                            currentToggleAllState: this.state.sheetOneToggleAll,
                            toggleAll: () => { this.toggleAll(MergeSheet.First); }
                        } })),
                react_1.default.createElement(MergeSheetAndKeySelection_1.default, { sheetNum: MergeSheet.Second, dfNames: this.state.originalDfNames, sheetIndex: this.state.sheetTwoIndex, otherSheetIndex: this.state.sheetOneIndex, setNewSheetIndex: (newSheetIndex) => { this.setNewSheetIndex(MergeSheet.Second, newSheetIndex); }, columnHeaders: this.props.sheetJSONArray[this.state.sheetTwoIndex].columns, mergeKey: this.state.sheetTwoMergeKey, setNewMergeKey: (newMergeKey) => this.setNewMergeKey(MergeSheet.Second, newMergeKey) }),
                react_1.default.createElement("div", null,
                    react_1.default.createElement("p", { className: 'default-taskpane-body-section-title-text' }, "Columns to Keep"),
                    react_1.default.createElement(MultiToggleBox_1.default, { itemsArray: sheetTwoColumns, itemToggleState: sheetTwoToggles, toggleItemAtIndex: (index) => this.toggleKeepColumnHeader(MergeSheet.Second, sheetTwoColumns[index]), toggleAllOptions: {
                            currentToggleAllState: this.state.sheetTwoToggleAll,
                            toggleAll: () => { this.toggleAll(MergeSheet.Second); }
                        } }))) }));
    }
}
exports.default = MergeTaskpane;
//# sourceMappingURL=MergeTaskpane.js.map