"use strict";
// Copyright (c) Mito
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const StepTypes_1 = require("../../../types/StepTypes");
// Icons
const ImportIcon_1 = __importDefault(require("../../icons/ImportIcon"));
const MergeIcon_1 = __importDefault(require("../../icons/MergeIcon"));
const PivotIcon_1 = __importDefault(require("../../icons/PivotIcon"));
const DeleteColumnIcon_1 = __importDefault(require("../../icons/DeleteColumnIcon"));
const AddColumnIcon_1 = __importDefault(require("../../icons/AddColumnIcon"));
const CustomDropdown_1 = __importDefault(require("../../elements/CustomDropdown"));
const DropdownIcon_1 = __importDefault(require("../../icons/DropdownIcon"));
const FilterIcons_1 = require("../../icons/FilterIcons");
const EditIcon_1 = __importDefault(require("../../icons/EditIcon"));
const MitoIcon_1 = __importDefault(require("../../icons/MitoIcon"));
/*
    Gets an icon for a specific step type, to display
    with that step
*/
function getStepIcon(stepType) {
    switch (stepType) {
        case StepTypes_1.StepType.Initialize: return (react_1.default.createElement(MitoIcon_1.default, null));
        case StepTypes_1.StepType.AddColumn: return (react_1.default.createElement(AddColumnIcon_1.default, null));
        case StepTypes_1.StepType.DeleteColumn: return (react_1.default.createElement(DeleteColumnIcon_1.default, null));
        case StepTypes_1.StepType.RenameColumn: return (react_1.default.createElement(EditIcon_1.default, null));
        case StepTypes_1.StepType.ReorderColumn: return (react_1.default.createElement(EditIcon_1.default, null));
        case StepTypes_1.StepType.FilterColumn: return (react_1.default.createElement(FilterIcons_1.FilterEmptyIcon, { height: '20', width: '20' }));
        case StepTypes_1.StepType.SetColumnFormula: return (react_1.default.createElement("div", { className: 'step-taskpane-missing-icon' }, "Fx"));
        case StepTypes_1.StepType.DataframeDelete: return (react_1.default.createElement(DeleteColumnIcon_1.default, null));
        case StepTypes_1.StepType.DataframeDuplicate: return (react_1.default.createElement(EditIcon_1.default, null));
        case StepTypes_1.StepType.DataframeRename: return (react_1.default.createElement(EditIcon_1.default, null));
        case StepTypes_1.StepType.SimpleImport: return (react_1.default.createElement(ImportIcon_1.default, null));
        case StepTypes_1.StepType.RawPythonImport: return (react_1.default.createElement(ImportIcon_1.default, null));
        case StepTypes_1.StepType.Sort: return (react_1.default.createElement(EditIcon_1.default, null));
        case StepTypes_1.StepType.Pivot: return (react_1.default.createElement(PivotIcon_1.default, { width: '20' }));
        case StepTypes_1.StepType.Merge: return (react_1.default.createElement(MergeIcon_1.default, { width: '27' }));
        default: return (react_1.default.createElement(EditIcon_1.default, null));
    }
}
exports.getStepIcon = getStepIcon;
/*
    An element in a list that displays information about a step, and
    eventually will allow the user to interact with that step (e.g.
    to start editing it).
*/
function StepDataElement(props) {
    const rollBackToStep = () => {
        props.setOpenedDropdown(false);
        void props.mitoAPI.checkoutStepByIndex(props.stepData.step_idx);
    };
    return (react_1.default.createElement("div", { className: 'step-taskpane-step-container' },
        react_1.default.createElement("div", { className: 'step-taskpane-step-container-left', style: { opacity: props.beforeCurrIdx ? '1' : '.5' }, onClick: rollBackToStep },
            react_1.default.createElement("div", { className: 'step-taskpane-step-icon' }, getStepIcon(props.stepData.step_type)),
            react_1.default.createElement("div", { className: 'step-taskpane-step-text' },
                react_1.default.createElement("div", { className: 'step-taskpane-step-header' }, props.stepData.step_display_name),
                react_1.default.createElement("div", { className: 'step-taskpane-step-subtext' }, props.stepData.step_description))),
        react_1.default.createElement("div", { className: 'step-taskpane-step-container-right', onClick: () => {
                props.setOpenedDropdown(!props.openedDropdown);
            } },
            react_1.default.createElement(DropdownIcon_1.default, null)),
        props.openedDropdown &&
            react_1.default.createElement(CustomDropdown_1.default, { closeDropdown: () => { props.setOpenedDropdown(false); } },
                react_1.default.createElement("div", { onClick: rollBackToStep }, "View this step"),
                react_1.default.createElement("div", { className: 'custom-dropdown-inactive' }, "Edit this step (coming soon)"))));
}
exports.default = StepDataElement;
//# sourceMappingURL=StepDataElement.js.map