"use strict";
// Copyright (c) Mito
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
// Import css
require("../../../css/tour.css");
const XIcon_1 = __importDefault(require("../icons/XIcon"));
const Tours_1 = require("./Tours");
const locationToClassNamesMapping = {
    [Tours_1.TourPopupLocation.BOTTOM_LEFT]: 'tour-container tour-container-bottom-left',
    [Tours_1.TourPopupLocation.BOTTOM_RIGHT]: 'tour-container tour-container-bottom-right',
    [Tours_1.TourPopupLocation.TOP_LEFT]: 'tour-container tour-container-top-left',
    [Tours_1.TourPopupLocation.TOP_RIGHT]: 'tour-container tour-container-top-right',
};
const Tour = (props) => {
    const [stepNumber, setStepNumber] = react_1.useState(0);
    const [skippedTour, setSkippedTour] = react_1.useState(false);
    // Construct full list of steps to show by appending the steps of each tour
    // we want to display together. 
    const steps = [];
    props.tourNames.forEach((tourName) => {
        steps.push(...Tours_1.tours[tourName]);
    });
    // Log the beginning of the tour. Because the useEffect has [] as
    // its second param, it only runs once. 
    react_1.useEffect(() => {
        void props.mitoAPI.sendLogMessage('begin_tour', {
            'tour_names': props.tourNames,
            'total_number_of_tour_steps': steps.length
        });
    }, []);
    /*
        Each tour is able to register actions that occur at the beginning of specific steps.
        The useEffect is fired each time the stepNumber changes.
    */
    react_1.useEffect(() => {
        if (steps[stepNumber].tourName === Tours_1.TourName.PIVOT && steps[stepNumber].stepNumber === 1) {
            props.setHighlightPivotTableButton(true);
        }
        else if (steps[stepNumber].tourName === Tours_1.TourName.COLUMN_FORMULAS && steps[stepNumber].stepNumber === 1) {
            props.setHighlightAddColButton(true);
        }
        else {
            props.setHighlightPivotTableButton(false);
            props.setHighlightAddColButton(false);
        }
    }, [stepNumber]);
    // Go to stepNumber if it exists, otherwise close the tour
    const goToStep = (newStepNumber) => {
        if (newStepNumber <= steps.length - 1) {
            // Log switching steps
            void props.mitoAPI.sendLogMessage('switched_tour_step', {
                'old_tour_name': steps[stepNumber].tourName,
                'old_tour_step': stepNumber,
                'new_tour_name': steps[newStepNumber].tourName,
                'new_tour_step': newStepNumber,
                'tour_names': props.tourNames,
                'total_number_of_tour_steps': steps.length
            });
            // Update the step number
            setStepNumber(newStepNumber);
        }
        else {
            void closeTour();
        }
    };
    // Called if you close the tour before you are on the final step
    const closeTourEarly = () => __awaiter(void 0, void 0, void 0, function* () {
        void props.mitoAPI.sendLogMessage('closed_tour_early', {
            'tour_names': props.tourNames,
            'tour_name': steps[stepNumber].tourName,
            'relative_tour_step_number': steps[stepNumber].stepNumber,
            'absolute_tour_step_number': stepNumber,
            'total_number_of_tour_steps': steps.length // Log the length of the entire tour if they had taken it all
        });
        // Mark that the user skipped the tour so we don't log a finished_tour event
        setSkippedTour(true);
        // Display the final tutorial step
        setStepNumber(steps.length - 1);
    });
    // Called if you close the tour on the final step
    const closeTour = () => __awaiter(void 0, void 0, void 0, function* () {
        // Log finishing the tour, if you didn't skip to the final step
        if (!skippedTour) {
            void props.mitoAPI.sendLogMessage('finished_tour', {
                'tour_names': props.tourNames,
                'total_number_of_tour_steps': steps.length
            });
        }
        // Make sure that the pivot & Add Col button is not stuck in pulsing mode. 
        props.setHighlightPivotTableButton(false);
        props.setHighlightAddColButton(false);
        // Send the closeTour message to the backend. We still record each tour individually
        // so we can later add new tours to display to users
        yield props.mitoAPI.sendCloseTour(props.tourNames);
    });
    /*
        Retrieve the finalStepText, which is the text displayed in the body of the tour popup.
        Although not enforced by the type system, a tour step should either have a defined stepText of stepTextFunction.
    */
    const stepText = steps[stepNumber].stepText;
    const stepTextFunction = steps[stepNumber].stepTextFunction;
    const finalStepText = stepText || (stepTextFunction && stepTextFunction(props.sheetJSON.columns[0]));
    // If we're at a valid step number
    return (react_1.default.createElement("div", { className: 'tour-container ' + locationToClassNamesMapping[steps[stepNumber].location], key: stepNumber },
        react_1.default.createElement("div", { className: 'tour-header-container', style: { backgroundColor: steps[stepNumber].stepHeaderBackgroundColor } },
            steps[stepNumber].stepHeader,
            react_1.default.createElement("div", { className: 'tour-header-close-button', onClick: () => __awaiter(void 0, void 0, void 0, function* () {
                    if (stepNumber >= steps.length - 1) {
                        yield closeTour();
                    }
                    else {
                        yield closeTourEarly();
                    }
                }) },
                react_1.default.createElement(XIcon_1.default, null))),
        react_1.default.createElement("div", { className: 'tour-step-text-container' }, finalStepText),
        react_1.default.createElement("div", { className: 'tour-button-container' },
            steps[stepNumber].displayBackButton &&
                react_1.default.createElement("div", { className: 'tour-close-button', onClick: () => goToStep(stepNumber - 1) }, "Back"),
            react_1.default.createElement("div", { className: 'tour-continue-button', onClick: () => goToStep(stepNumber + 1) }, steps[stepNumber].advanceButtonText)),
        react_1.default.createElement("div", { className: 'tour-progress-bar-container' },
            react_1.default.createElement("p", { className: 'tour-progress-bar-text' },
                stepNumber + 1,
                "/",
                steps.length))));
};
exports.default = Tour;
//# sourceMappingURL=Tour.js.map